VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
  Persistable = 0  'NotPersistable
  DataBindingBehavior = 0  'vbNone
  DataSourceBehavior  = 0  'vbNone
  MTSTransactionMode  = 0  'NotAnMTSObject
END
Attribute VB_Name = "clsApplication"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = False
Option Explicit

' Copyright (C) 2006 Kristian S. Stangeland

' This program is free software; you can redistribute it and/or
' modify it under the terms of the GNU General Public License
' as published by the Free Software Foundation; either version 2
' of the License, or (at your option) any later version.

' This program is distributed in the hope that it will be useful,
' but WITHOUT ANY WARRANTY; without even the implied warranty of
' MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
' GNU General Public License for more details.

' You should have received a copy of the GNU General Public License
' along with this program; if not, write to the Free Software
' Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

' All the settings of this application
Public Settings As New clsSettings

' All current plugins
Public Plugins As New Collection

Public Property Get FileSystem() As clsFileOperation

    ' Return this class
    Set FileSystem = FileOperation

End Property

Public Property Get ActiveForms() As Collection

    ' Return the collection of open forms
    Set ActiveForms = Forms

End Property

Public Function FindForm(Value, Optional sProperty As String = "PrivateName") As Form

    On Error Resume Next
    Dim Form As Form, vValue
    
    ' Move through all open forms
    For Each Form In Forms
    
        ' Retrieve the property
        vValue = CallByName(Form, sProperty, VbGet)
    
        ' Test the property of the form towards the value
        If SafeEqual(vValue, Value) Then
        
            ' Return the form
            Set FindForm = Form
        
            ' We're done
            Exit Function
        
        End If
    
    Next

End Function

Public Property Get MainForm(refComm As clsCommunication) As Form

    Dim Form As Object
    
    ' Loop through all current forms
    For Each Form In Forms
    
        ' Make sure this is a window form
        If TypeOf Form Is frmWindow Then
    
            ' See firstly if the form is the main window
            If Not Form.PrivateSession Then
                
                ' Then check the communcication class
                If Form.refComm Is refComm Then
                
                    ' Return this form
                    Set MainForm = Form
                    
                    ' We're done
                    Exit Property
                
                End If
            
            End If
    
        End If
    
    Next

End Property

Private Function SafeEqual(Value1, Value2) As Boolean

    ' See whether or not one of the variables are a object
    If IsObject(Value1) Or IsObject(Value2) Then
        SafeEqual = (Value1 Is Value2)
    Else
        SafeEqual = (Value1 = Value2)
    End If

End Function

Public Sub ExitApp()

    ' Attempt to reset the app
    ResetApp
    
    ' Exit the application immediently
    End

End Sub

Public Sub ResetSettings()

    ' Remove all current settings
    DeleteSetting "ChatApp"
    
End Sub

Public Sub ResetApp()

    Dim Form As Form

    ' Go through all forms
    For Each Form In Forms
    
        If Not (Form Is frmMain) Then
            Unload Form
        End If
    
    Next
    
    ' Show the logon from again
    frmLogon.Show

End Sub

Public Sub LoadPlugins()

    On Error Resume Next
    Dim sFile As Variant, Plugin As Object, strClassName As String
    
    ' Go through all executable files in the plugin-folder
    For Each sFile In FileOperation.RetriveFileList(FileOperation.ValidPath(App.Path) & "Plugins\", "*.dll", False, vbNormal)
    
        Select Case FileOperation.GetFileExtension(CStr(sFile))
        Case "dll"

            ' Clear all errors
            Err.Clear
            
            ' How the class is registered in the registry
            strClassName = FileOperation.GetNoExtension(FileOperation.GetFileName(CStr(sFile))) & ".PluginMain"
            
            ' Try to create the object
            Set Plugin = CreateObject(strClassName)
    
            If Err = 429 Then ' ERROR: ActiveX component can't create object
                
                ' Clear the error
                Err.Clear
                
                ' Try to register the object
                Shell "regsvr32 /s " & Chr(34) & CStr(sFile) & Chr(34)
                
                ' Load the plugin again
                Set Plugin = CreateObject(strClassName)
            
            End If
    
            ' Now, if this didn't work either, it could be something wrong with the registration of the DLL, so we'll retry
            If Err = 429 Then
            
                ' Firstly, clear the error
                Err.Clear
                
                ' Then, unregister the DLL
                Shell "regsvr32 /u /s " & Chr(34) & CStr(sFile) & Chr(34)
                
                ' And register it again
                Shell "regsvr32 /s " & Chr(34) & CStr(sFile) & Chr(34)
            
                ' Try to load the plugin for the last time
                Set Plugin = CreateObject(strClassName)
            
            End If
            
            ' If even this dosen't work, we'll just ignore the whole thing
            If Err <> 429 Then
    
                ' See if this plugin actually is allowed
                If Settings.PluginEnabled(Plugin.Name) Then
        
                    ' Add plugin
                    Plugins.Add Plugin, Plugin.Name
                
                    ' Initialize plugin
                    Plugin.Initialize Me
                
                End If

            End If

        End Select
        
    Next

End Sub

