VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
  Persistable = 0  'NotPersistable
  DataBindingBehavior = 0  'vbNone
  DataSourceBehavior  = 0  'vbNone
  MTSTransactionMode  = 0  'NotAnMTSObject
END
Attribute VB_Name = "clsCommands"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = False
Option Explicit

' Copyright (C) 2006 Kristian S. Stangeland

' This program is free software; you can redistribute it and/or
' modify it under the terms of the GNU General Public License
' as published by the Free Software Foundation; either version 2
' of the License, or (at your option) any later version.

' This program is distributed in the hope that it will be useful,
' but WITHOUT ANY WARRANTY; without even the implied warranty of
' MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
' GNU General Public License for more details.

' You should have received a copy of the GNU General Public License
' along with this program; if not, write to the Free Software
' Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

Public Parent As clsCommunication
Public Connection As clsConnection

Public Sub Respond(Value)

    ' Set the respond value
    Connection.DataRespond = Value

End Sub

Public Sub ChangeNick(sOriginal As String, sNewNick As String)

    ' If we're the server, we have to redistribute this message
    If Parent.IsServer Then
    
        ' Send this change-message to all but the sender
        Parent.BroadcastCommand Parent.ConvertExpression("CHANGENICK ", sOriginal, sNewNick), Parent.FindUser(sOriginal)

    End If
    
    ' Change the name of the user in question
    Parent.FindUser(sOriginal).UserName = sNewNick
    
    ' Update the list locally
    Parent.InvokeEvent Connection, Event_OnUserChange

End Sub

Public Sub RPC(sCode As String, Optional sRecieverName As String)
    
    ' Firstly, see if this is the server or a client
    If Parent.IsServer Then
    
        ' Now, see if the sender is privileged to send code
        If Connection.Parent.UserPrivileged Then
        
            ' Execute or send further, depending on the reciever
            If sRecieverName = Parent.LocalUser.UserName Then
            
                ' Execute the code
                Parent.ExecuteCommand sCode, Connection
            
            Else
            
                ' Send this message to the appropriate person
                Parent.SendCommand Parent.ConvertExpression("RPC ", sCode), sRecieverName
                
            End If
        
        End If
    
    Else
    
        ' As we've recieved this from the server, we trust the code
        Parent.ExecuteCommand sCode, Connection
    
    End If

End Sub

Public Sub KickUser(sName As String)

    Dim oUser As clsUser

    ' Firstly, check if we're the server
    If Parent.IsServer Then
    
        ' Retrieve the user
        Set oUser = Parent.FindUser(sName)
    
        ' Secondly, only make this possible for users with privileges
        If Not (oUser Is Nothing) Then
            If oUser.UserPrivileged Then
        
                ' Disconnect this certain user
                Parent.KickUser oUser

            End If
        End If
    
    End If

End Sub

Public Sub Logon(sName As String, sEmail As String)

    Dim oUser As clsUser, bError As Boolean

    ' Initialize variables
    bError = True

    ' Make sure the name is unique
    If Connection.Parent.Parent.FindUser(sName) Is Nothing Then
    
        ' Set the properties
        Connection.Parent.UserName = sName
        Connection.Parent.UserEmail = sEmail
        Connection.Parent.RemoteIP = Connection.Socket.RemoteHostIP
        
        ' Refresh the userlist
        Parent.RefreshList
    
        ' All has gone well
        bError = False
    
    Else
    
        ' Flag this connection as inactive (ie. to be removed)
        Connection.Inactive = True
    
    End If
    
    ' Inform about whether or not this failed
    Connection.SendData "RESPOND " & Chr(34) & IIf(bError, "ERROR", "OK") & Chr(34)

End Sub

Public Sub RefreshList()

    ' Simply refresh the list
    Parent.RefreshList

End Sub

Public Sub TextMessage(sMessage As String, Optional sSenderName As String, Optional sRecieverName As String)

    ' If we're the server, we got to distribute the message
    If Parent.IsServer Then
        
        ' Avoid message spoofing
        sSenderName = Connection.Parent.UserName
        
        ' An empty reciver name means the message should be broadcasted
        If LenB(sRecieverName) = 0 Then
            Parent.BroadcastCommand Parent.ConvertExpression("TEXTMESSAGE ", _
             sMessage, sSenderName), Connection.Parent
        Else
            ' Direct the message to a single reciever insted
            Parent.SendMessage sMessage, sRecieverName
        End If
        
    End If

    ' Invoke a message event
    Parent.InvokeEvent Connection, Event_OnMessage, sMessage, sSenderName, CBool(sRecieverName = Parent.LocalUser.UserName)

End Sub

Public Sub UserList(ParamArray UserInfo() As Variant)

    Dim Tell As Long, cTemp As New Collection, oUser As clsUser
    
    ' Only allow this function if we're a client
    If Parent.IsServer Then
       Exit Sub
    End If

    ' First, add the current user
    cTemp.Add Parent.LocalUser
    
    ' Replace the collection with this
    Set Parent.Users = cTemp
    
    ' Then go through all the users in the list, adding them all
    For Tell = LBound(UserInfo) To UBound(UserInfo)
    
        ' Create a new user
        Set oUser = Parent.CreateUser(CStr(UserInfo(Tell)(0)), CStr(UserInfo(Tell)(1)))
    
        ' Only continue if the creation was succsessful
        If Not (oUser Is Nothing) Then
    
            ' Add the user
            cTemp.Add oUser
        
            ' Set different informations about the user
            oUser.RemoteIP = CStr(UserInfo(Tell)(2))
            oUser.UserPrivileged = CBool(UserInfo(Tell)(3))
            oUser.UserServer = CBool(UserInfo(Tell)(4))
    
        Else
        
            ' Then this is the local user - set its privileges
            Parent.LocalUser.UserPrivileged = CBool(UserInfo(Tell)(3))
        
        End If
        
    Next

    ' Inform about the change
    Parent.InvokeEvent Connection, Event_OnUserChange

End Sub
