VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
  Persistable = 0  'NotPersistable
  DataBindingBehavior = 0  'vbNone
  DataSourceBehavior  = 0  'vbNone
  MTSTransactionMode  = 0  'NotAnMTSObject
END
Attribute VB_Name = "clsCommunication"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = False
Option Explicit

' Copyright (C) 2006 Kristian S. Stangeland

' This program is free software; you can redistribute it and/or
' modify it under the terms of the GNU General Public License
' as published by the Free Software Foundation; either version 2
' of the License, or (at your option) any later version.

' This program is distributed in the hope that it will be useful,
' but WITHOUT ANY WARRANTY; without even the implied warranty of
' MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
' GNU General Public License for more details.

' You should have received a copy of the GNU General Public License
' along with this program; if not, write to the Free Software
' Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

'Public Event FileProgress(ID As Long, Position As Long, MaxLenght As Long)
'Public Event FileTransfer(Sender As clsUser, FileName As String, ID As Long)
Public Event MessageRecieved(Sender As clsUser, sMessage As String, bPrivate As Boolean)
Public Event UserListChanged(UserLeft, UserAdded)
Public Event Disconnected()
Public Event Connected()

' Initialize different variables
Public Users As New Collection

' A buffer used to determine what users has left and been added recently
Private aUsers As Variant

' The script engine to use
Private objScript As ScriptControl

' Whether or not we're acting as a server or client
Private bServer As Boolean

Public Property Get ServerAddress() As String

    ' This is achieved differently depending on if we're a client or server
    If bServer Then
        ServerAddress = LocalUser.Connection.Socket.LocalIP
    Else
        ServerAddress = LocalUser.Connection.Socket.RemoteHost
    End If

End Property

Public Function CreateLog(sFile As String) As clsLogging

    ' Firstly, create a new, fresh object
    Set CreateLog = New clsLogging
    
    ' Then, initialize the object
    CreateLog.CreateLog sFile, Me

End Function

Public Function CreateUser(Optional sName As String, Optional sEmail As String) As clsUser

    ' Firstly, check if there's a similar user already logged on
    If Not (FindUser(sName) Is Nothing) Then
        Exit Function
    End If

    ' Create a new user
    Set CreateUser = New clsUser

    ' Refer the parent to the current module
    Set CreateUser.Parent = Me
    Set CreateUser.Connection.Parent = CreateUser

    ' Set the different properties
    CreateUser.UserName = sName
    CreateUser.UserEmail = sEmail

End Function

Public Function Connect(oCurrUser As clsUser, ByVal sAddress As String, ByVal lngPort As Long) As Long

    ' Clear all connections
    ClearAll
    
    ' In this case we're acting like a client
    bServer = False
    
    ' Append the current user class to the collection
    Users.Add oCurrUser
    
    ' Attempts to establish a connection to the server
    oCurrUser.LocalUser = True
    oCurrUser.RemoteIP = "127.0.0.1"
    oCurrUser.Connection.Connect sAddress, lngPort

    ' Wait for a respond
    WaitForRespond oCurrUser.Connection
    
    ' Return whether or not the connection was succsessful
    If oCurrUser.Connection.DataRespond <> "OK" Then
        Connect = -1
    End If

End Function

Public Sub Listen(oCurrUser As clsUser, lngPort As Long)

    ' Clear all connections
    ClearAll
    
    ' In this case we're the server
    bServer = True
    
    ' Append the current user - the server - to the collection
    Users.Add oCurrUser
        
    ' Start listening for connection requests
    oCurrUser.UserPrivileged = True
    oCurrUser.LocalUser = True
    oCurrUser.RemoteIP = "127.0.0.1"
    oCurrUser.Connection.Listen lngPort
    
    ' Inform that the userlist has changed
    InvokeEvent Nothing, Event_OnUserChange

End Sub

Public Sub ChangeNick(sNewNick As String)

    Dim sExpression As String

    ' Firstly, make sure no-one else has this nick
    If FindUser(sNewNick) Is Nothing Then
    
        ' The message to send
        sExpression = ConvertExpression("CHANGENICK ", LocalUser.UserName, sNewNick)
    
        ' Proceed depending on whether or not we're the server,
        If bServer Then
        
            ' Broadcast this change
            BroadcastCommand sExpression
        
        Else
        
            ' Inquire the server to broadcast the message for us
            SendCommand sExpression
        
        End If
        
        ' Change the nick locally
        LocalUser.UserName = sNewNick
        
        ' Update the nick-list
        InvokeEvent LocalUser.Connection, Event_OnUserChange
    
    End If

End Sub

Public Sub KickUser(oUser As clsUser)

    ' See if we're the server or a client
    If bServer Then
    
        ' If we're the server, just remove the user
        RemoveUser oUser
    
        ' Also, update the user list
        RefreshList
    
    Else
    
        ' Command the server to kick a user (will only work with privileges)
        SendCommand "KICKUSER " & oUser.UserName
    
    End If

End Sub

Private Sub RemoveUser(oUser As Object)

    Dim tempCol As New Collection, User As Object

    ' Add all users beside the given user to remove
    For Each User In Users
        If Not (oUser Is User) Then
            ' Add the user
            tempCol.Add User
        End If
    Next
    
    ' Clear all references in the user-class
    oUser.ClearAll
    
    ' Replace the user collection
    Set Users = tempCol

End Sub

Public Sub SendRPC(sCode As String, Optional sRecieverName As String)

    ' Send the RPC-command
    SendCommand ConvertExpression("RPC ", sCode), sRecieverName
    
End Sub

Public Sub SendCommand(sCommand As String, Optional sRecieverName As String)

    Dim oUser As clsUser, oConnection As clsConnection

    ' Regardless of connection type - if we're the reciever, it is simple
    If sRecieverName = LocalUser.UserName Then
    
        ' Execute the command
        ExecuteCommand sCommand, LocalUser.Connection
    
        ' No need to continue
        Exit Sub
        
    End If

    ' If we're a server, we DO need a reciever name
    If bServer Then
    
        ' Attempt to retrieve a user
        Set oUser = FindUser(sRecieverName)
        
        ' See if this was successful
        If oUser Is Nothing Then
        
            ' No user with this name - exit
            Exit Sub
        
        Else
        
            ' Set the connection
            Set oConnection = oUser.Connection

        End If
    
    Else
    
        ' If we're a client, it is simple
        Set oConnection = LocalUser.Connection
    
    End If
    
    ' Finally send the command untouched
    oConnection.SendData sCommand

End Sub

Public Sub SendMessage(sText As String, Optional sRecieverName As String)

    ' Ignore possible errors (i.e. user not found)
    On Error Resume Next

    ' If the reciever corresponds to the current user, ...
    If sRecieverName = LocalUser.UserName Then
    
        ' Invoke the event
        InvokeEvent LocalUser.Connection, Event_OnMessage, sText, LocalUser.UserName, True

        ' No need to continue
        Exit Sub
    
    End If

    ' Send the message differently depending on the connection type
    If bServer Then
        
        ' As usual, an empty string means broadcasting
        If LenB(sRecieverName) = 0 Then
            BroadcastCommand ConvertExpression("TEXTMESSAGE ", sText, LocalUser.UserName)
        Else
            ' If not, send the message directly
            FindUser(sRecieverName).Connection.SendData ConvertExpression("TEXTMESSAGE ", _
             sText, LocalUser.UserName, sRecieverName)
        End If
    
    Else
        LocalUser.Connection.SendData ConvertExpression("TEXTMESSAGE ", sText, LocalUser.UserName, sRecieverName)
    End If

End Sub

Public Sub ClearAll()

    ' Clear the current user state
    Set Users = New Collection
    
End Sub

Public Sub BroadcastCommand(sMessage As String, Optional Exception As clsUser)

    Dim oUser As clsUser
    
    ' Go through all users, ...
    For Each oUser In Users
        
        ' ... except a certain user, ...
        If (Not (oUser Is Exception)) And oUser.LocalUser = False Then
        
            ' ... sending the same message
            oUser.Connection.SendData sMessage
    
        End If
    
    Next

End Sub

Public Function FindUser(Value As Variant, Optional sProperty As String = "UserName") As clsUser

    Dim oUser As clsUser
    
    ' Run through the collection of users, ...
    For Each oUser In Users
    
        ' ... returning the user if the string corresponds to the property
        If CallByName(oUser, sProperty, VbGet) = Value Then
            
            ' Return user
            Set FindUser = oUser
            
            ' We're done - exit prosedure
            Exit Function
            
        End If
    
    Next

End Function

Public Property Get LocalUser() As clsUser

    ' Return the local user
    Set LocalUser = FindUser(True, "LocalUser")

End Property

Public Sub RefreshList()

    Dim oUser As Object, sTemp As String

    ' The update of the list is accieved differently depending on the connection type
    If bServer Then
    
        ' Illiterate through all users, ...
        For Each oUser In Users
        
            ' ... adding the information of each
            sTemp = sTemp & IIf(LenB(sTemp) > 0, ", ", "") & ConvertExpression("Array(", _
             oUser.UserName, oUser.UserEmail, oUser.RemoteIP, oUser.UserPrivileged, oUser.LocalUser) & ")"
        
        Next
    
        ' Broadcast the command
        BroadcastCommand "USERLIST " & sTemp
        
        ' A changed has been made to the userlist, apperently
        InvokeEvent LocalUser.Connection, Event_OnUserChange
    
    Else
    
        ' Request a full refresh of the userlist
        LocalUser.Connection.SendData "REFRESHLIST"
    
    End If

End Sub

Public Sub InvokeEvent(oConnection As clsConnection, eEvent As ConnectionEvents, ParamArray Paramenters() As Variant)

    Dim oUser As clsUser, tempClass As New Collection, aAdded, aLeft
    
    ' Retrieve the local user
    Set oUser = LocalUser

    ' Execute an action based upon the event invoked
    Select Case eEvent
    Case Event_OnClose
        
        If bServer Then

            ' Simply remove the user from the collection
            RemoveUser oConnection.Parent
            
            ' Update the user-list
            RefreshList

        Else
        
            ' If the client gets disconnected, inform about this
            RaiseEvent Disconnected
        
        End If
    
    Case Event_OnConnect
    
        ' When a client has connected, ...
        If Not bServer Then
        
            ' ... send more detalied information about the user
            oConnection.SendData ConvertExpression("LOGON ", oUser.UserName, oUser.UserEmail)
        
        End If
        
        ' We have connected
        RaiseEvent Connected
    
    Case Event_OnConnectionRequest
    
        ' Naturally, this event can only be invoked as a server
        If bServer Then
            
            ' Create a new user (and thus connection) ...
            Set oUser = CreateUser
            
            ' ... to accept this request
            oUser.Connection.Socket.Accept CLng(Paramenters(0))
        
            ' Add the user to the list
            Users.Add oUser
        
        End If
    
    Case Event_OnDataArrival
    
        ' Execute the incomming data
        ExecuteCommand "Command." & oConnection.DataBuffer, oConnection

    Case Event_OnMessage
    
        ' Invoke an event if an message has been sent
        If Paramenters(1) <> "" Then
            RaiseEvent MessageRecieved(FindUser(Paramenters(1)), CStr(Paramenters(0)), CBool(Paramenters(2)))
        Else
            RaiseEvent MessageRecieved(oConnection.Parent, CStr(Paramenters(0)), CBool(Paramenters(2)))
        End If
    
    Case Event_OnUserChange
        
        ' The user list has changed - inform about this
        RaiseEvent UserListChanged(CompareArray(ConvertArray(Users), aUsers), CompareArray(aUsers, _
                    ConvertArray(Users)))
    
        ' Now, save this version of the list
        aUsers = ConvertArray(Users)
    
    End Select

End Sub

Private Function CompareArray(refArray, newArray) As String

    Dim Tell As Long, lngValue As Long, lngCount As Long, bResult As Boolean
    
    ' If there's nothing to compare againts, just return the array
    If Not IsArray(refArray) Then
        CompareArray = newArray(LBound(newArray))
        Exit Function
    ElseIf Not IsArray(newArray) Then
        Exit Function
    End If
    
    ' Go through the arrays, comparing each element
    For Tell = LBound(newArray) To UBound(newArray)
        
        ' Reinitialize variables
        bResult = False
        
        ' See if we can't find a match
        For lngValue = LBound(refArray) To UBound(refArray)
            If newArray(Tell) = refArray(lngValue) Then
            
                ' The match are positive
                bResult = True
                
                ' We're done
                Exit For
            
            End If
        Next
        
        ' See if it was successful and return the value
        If Not bResult Then
            CompareArray = newArray(Tell)
        End If

    Next

End Function

Private Function ConvertArray(refCollection As Collection) As Variant

    Dim Tell As Long, aResult
    
    ' Firstly, redim an array
    ReDim aResult(1 To refCollection.Count)

    ' Populate the array
    For Tell = 1 To refCollection.Count
        aResult(Tell) = refCollection(Tell).UserName
    Next
    
    ' Return the array
    ConvertArray = aResult

End Function

Public Sub ExecuteCommand(sCommand As String, Connection As clsConnection)

    On Error GoTo InUse
    Dim objCommands As New clsCommands, refClass As ScriptControl

    ' Use the default engine
    Set refClass = objScript

    ' Set the parent
    Set objCommands.Parent = Me
    Set objCommands.Connection = Connection

    ' Append the different controls
    refClass.Reset
    refClass.AddObject "Command", objCommands
    refClass.AddObject "Connection", Connection
    refClass.AddObject "Communication", Me
    refClass.AddObject "App", ChatApp

    ' Execute the line
    refClass.AddCode sCommand

Exit Sub
InUse:

    ' If the object is already in use
    If Err = -2147467259 Then
    
        ' Create a new script class
        Set refClass = New ScriptControl
        
        ' Initialize the new class
        refClass.Language = "VBScript"
    
        ' Attempt again
        Resume
    
    End If

End Sub

Public Function WaitForRespond(oConnection As clsConnection) As String

    ' Firstly, clear the respond variable
    oConnection.DataRespond = ""

    ' Then loop until a respond is given
    Do Until oConnection.DataRespond <> ""
        Sleep 10
        DoEvents
    Loop
    
    ' Return the respond
    WaitForRespond = oConnection.DataRespond

End Function

Private Function GetArrayLenght(aArray() As Byte) As Long

    ' Return the lenght of the array
    GetArrayLenght = UBound(aArray) - LBound(aArray) + 1

End Function

Friend Function ConvertExpression(ByVal sBegin As String, ParamArray Paramenters() As Variant) As String

    Dim Tell As Long
    
    ' Go through all given paramenters
    For Tell = LBound(Paramenters) To UBound(Paramenters)

        ' Fix a bug
        If varType(Paramenters(Tell)) = vbBoolean Then
            Paramenters(Tell) = ToBool(CBool(Paramenters(Tell)))
        End If

        ' Append each element, depending on the type of the string
        If IsNumeric(Paramenters(Tell)) Then
            sBegin = sBegin & IIf(Tell > LBound(Paramenters), ", ", "") & Paramenters(Tell)
        Else
            sBegin = sBegin & IIf(Tell > LBound(Paramenters), ", ", "") & Chr(34) & _
             Replace(Replace(Paramenters(Tell), Chr(34), Chr(34) & " & Chr(34) & " & _
             Chr(34)), vbCrLf, Chr(34) & " & vbCrLf & " & Chr(34)) & Chr(34)
        End If

    Next
    
    ' Return the string
    ConvertExpression = sBegin

End Function

Public Property Get IsServer() As Boolean

    ' Return whether or not we're the server
    IsServer = bServer

End Property

Private Function ToBool(bValue As Boolean) As String

    ' Return the right string depending on the value
    Select Case bValue
    Case False: ToBool = "False"
    Case True: ToBool = "True"
    End Select

End Function

Private Sub Class_Initialize()

    ' Create a script control
    Set objScript = New ScriptControl
    
    ' Initialize language
    objScript.Language = "VBScript"

End Sub
