VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
  Persistable = 0  'NotPersistable
  DataBindingBehavior = 0  'vbNone
  DataSourceBehavior  = 0  'vbNone
  MTSTransactionMode  = 0  'NotAnMTSObject
END
Attribute VB_Name = "clsConnection"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = False
Option Explicit

' Copyright (C) 2006 Kristian S. Stangeland

' This program is free software; you can redistribute it and/or
' modify it under the terms of the GNU General Public License
' as published by the Free Software Foundation; either version 2
' of the License, or (at your option) any later version.

' This program is distributed in the hope that it will be useful,
' but WITHOUT ANY WARRANTY; without even the implied warranty of
' MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
' GNU General Public License for more details.

' You should have received a copy of the GNU General Public License
' along with this program; if not, write to the Free Software
' Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

' Used to optimize the allocation of the strings
Private Declare Function SysAllocStringByteLen Lib "oleaut32" (ByVal olestr As Long, ByVal BLen As Long) As Long

' Our data header all information is sent through
Private Type DataHeader
    intLenght As Integer
    intChannel As Integer
    lngOffset As Long
End Type

' All the events this connection may invoke
Public Enum ConnectionEvents
    Event_OnClose
    Event_OnConnect
    Event_OnConnectionRequest
    Event_OnDataArrival
    Event_OnMessage
    Event_OnUserChange
End Enum

' The CSocket connection object
Public WithEvents Socket As clsSocket
Attribute Socket.VB_VarHelpID = -1

' The parent object which recieves all the events
Public Parent As Object

' A buffer containing data before it can be reconstructed to a fully package
Public DataBuffer As String
Public DataChannel As Integer
Public DataLeft As Long
Public DataOffset As Long
Public DataRespond As Variant
Public Inactive As Boolean

Public Sub SendData(ByVal sData As String, Optional lngChannel As Long, Optional lngOffset As Long)

    Dim aData() As Byte

    ' Allocate enough space
    ReDim aData(1 To (8 + LenB(sData)))

    ' Build the data
    CopyMemory aData(1), LenB(sData), 2
    CopyMemory aData(3), lngChannel, 2
    CopyMemory aData(5), lngOffset, 4
    CopyMemory aData(9), ByVal StrPtr(sData), LenB(sData)

    ' Send the actual data
    Socket.SendData aData

End Sub

Public Sub Connect(ByVal sRemoteHost As String, ByVal lngPort As Long)
    
    ' Connect to the server
    If Socket.State = sckClosed Then
        Socket.Connect sRemoteHost, lngPort
    End If

End Sub

Public Sub CloseConnection()

    ' Close the current connection
    Socket.CloseSocket

    ' Then simply recreate the class
    Set Socket = New clsSocket

End Sub

Public Sub Listen(ByVal lngPort As Long)

    ' Start listening to the specified port
    If Socket.State = sckClosed Then
        Socket.LocalPort = lngPort
        Socket.Listen
    End If

End Sub

Private Sub Class_Initialize()

    ' Initialize the socket
    Set Socket = New clsSocket

End Sub

Private Sub Socket_OnClose()

    ' Inform about the current event
    Parent.Parent.InvokeEvent Me, Event_OnClose

End Sub

Private Sub Socket_OnConnect()

    ' Is invoked when the connection has been established
    Parent.Parent.InvokeEvent Me, Event_OnConnect

End Sub

Private Sub Socket_OnConnectionRequest(ByVal requestID As Long)

    ' Is invoked when a connection is requested
    Parent.Parent.InvokeEvent Me, Event_OnConnectionRequest, requestID

End Sub

Private Sub Socket_OnDataArrival(ByVal bytesTotal As Long)

    Dim aData() As Byte

    ' Retrieve the incomming information
    Socket.GetData aData, vbArray + vbByte
    
    ' Process this data
    ProcessData aData

End Sub

Private Sub ProcessData(aData() As Byte)

    Dim tHeader As DataHeader, sTemp As String, lngTemp As Long, lngAdded As Long
    
    ' See whether or not there's anything to process
    If GetArrayLenght(aData) < 1 Then
        Exit Sub
    End If
    
    ' Parse the information
    If DataLeft <= 0 Then
    
        ' Firstly, clear our buffer
        DataBuffer = ""
        
        ' Now, parse the header
        CopyMemory tHeader, aData(LBound(aData)), LenB(tHeader)
        
        ' Get the amount of data to allocate
        If GetArrayLenght(aData) - 8 > tHeader.intLenght Then
            lngTemp = tHeader.intLenght / 2
        Else
            lngTemp = (GetArrayLenght(aData) - 8) / 2
        End If
        
        ' Allocate enough data
        sTemp = SysAllocStringByteLen(vbNullString, lngTemp)
                
        ' Copy the remaining data onto the string
        CopyMemory ByVal StrPtr(sTemp), aData(8), lngTemp * 2
    
        ' Save the recieved data
        DataLeft = tHeader.intLenght - (GetArrayLenght(aData) - 8)
        DataChannel = tHeader.intChannel
        DataOffset = tHeader.lngOffset
        DataBuffer = sTemp
    
        ' See whether or not we have to wait for further data
        If GetArrayLenght(aData) - 8 >= tHeader.intLenght Then
        
            ' Inform our parent class that information has arrived
            Parent.Parent.InvokeEvent Me, Event_OnDataArrival
            
            ' See if there's more to parse
            If GetArrayLenght(aData) - 8 > tHeader.intLenght Then
            
                ' Process this data further
                ProcessData Mid(CStr(aData), 5 + (tHeader.intLenght / 2))
            
            End If
        
        End If
        
    Else
    
        ' Append the current data
        If DataLeft < GetArrayLenght(aData) Then
            DataBuffer = DataBuffer & Mid(CStr(aData), 1, DataLeft)
            lngAdded = DataLeft
        Else
            DataBuffer = DataBuffer & CStr(aData)
            lngAdded = GetArrayLenght(aData)
        End If
        
        ' Decrease the missing data counter
        DataLeft = DataLeft - lngAdded
        
        ' See whether or not the transmission is complete
        If DataLeft > 0 Then
            
            ' Yup, the information is here
            Parent.Parent.InvokeEvent Me, Event_OnDataArrival
            
            ' Now, see if there's more to process
            If lngAdded <> DataLeft Then
                ProcessData Mid(CStr(aData), lngAdded)
            End If
        
        End If
    
    End If

End Sub

Private Function GetArrayLenght(aArray() As Byte) As Long

    ' Return the lenght of the array
    GetArrayLenght = UBound(aArray) - LBound(aArray) + 1

End Function

